"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeQueries = exports.mapQuery = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const utils_1 = require("./utils");
/**
 * Creates a new `Query` which maps a source value by the provided mapping
 * function.
 *
 * @param query source query
 * @param mapper value mapper
 * @param options options for processing the result value
 */
function mapQuery(query, mapper, options) {
    const { shareReplayWithRef, buffer } = createShareReplayWithRef();
    let value$ = query.value$.pipe((0, operators_1.map)(mapper));
    value$ = distinctValue(value$, options === null || options === void 0 ? void 0 : options.distinct).pipe(shareReplayWithRef);
    function get() {
        return buffer.ref ? buffer.ref.value : mapper(query.get());
    }
    return { get, value$ };
}
exports.mapQuery = mapQuery;
/**
 * Creates a new `Query` which takes the latest values from source queries
 * and merges them into a single value.
 *
 * @param queries source queries
 * @param merger value merger
 * @param options options for processing the result value
 */
function mergeQueries(queries, merger, options) {
    const { shareReplayWithRef, buffer } = createShareReplayWithRef();
    let value$ = (0, rxjs_1.combineLatest)(queries.map((query) => query.value$)).pipe((0, operators_1.map)((values) => merger(...values)));
    value$ = distinctValue(value$, options === null || options === void 0 ? void 0 : options.distinct).pipe(shareReplayWithRef);
    function get() {
        if (buffer.ref) {
            return buffer.ref.value;
        }
        return merger(...queries.map((query) => query.get()));
    }
    return { get, value$ };
}
exports.mergeQueries = mergeQueries;
function distinctValue(value$, distinct) {
    var _a, _b;
    if (distinct === false) {
        return value$;
    }
    const comparator = (_a = (distinct === true ? undefined : distinct === null || distinct === void 0 ? void 0 : distinct.comparator)) !== null && _a !== void 0 ? _a : utils_1.DEFAULT_COMPARATOR;
    const keySelector = (_b = (distinct === true ? undefined : distinct === null || distinct === void 0 ? void 0 : distinct.keySelector)) !== null && _b !== void 0 ? _b : rxjs_1.identity;
    return value$.pipe((0, operators_1.distinctUntilChanged)(comparator, keySelector));
}
function createShareReplayWithRef() {
    const buffer = {};
    const shareReplayWithRef = (source$) => source$.pipe((0, operators_1.tap)({
        next: (value) => (buffer.ref = { value }),
        unsubscribe: () => (buffer.ref = undefined),
    }), (0, rxjs_1.shareReplay)({ bufferSize: 1, refCount: true }));
    return { shareReplayWithRef, buffer };
}
//# sourceMappingURL=queryMappers.js.map