import { Observable, OperatorFunction, Subscription } from 'rxjs';
import { Action } from './action';
import { Controller } from './controller';
import { EffectState } from './effectState';
import { Query } from './query';
/**
 * Handler for an event. It can be asynchronous.
 *
 * @result a result, Promise or Observable
 */
export type EffectHandler<Event, Result> = (event: Event) => Result | Promise<Result> | Observable<Result>;
export type EffectEventProject<Event, Result> = (event: Event) => Observable<Result>;
export type EffectPipeline<Event, Result> = (eventProject: EffectEventProject<Event, Result>) => OperatorFunction<Event, Result>;
export type EffectOptions<Event, Result> = Readonly<{
    /**
     * Custom pipeline for processing effect's events.
     *
     * `mergeMap` pipeline is used by default.
     */
    pipeline?: EffectPipeline<Event, Result>;
}>;
/**
 * Effect encapsulates a handler for Action or Observable.
 *
 * It provides the state of execution results, which can be used to construct
 * a graph of business logic.
 *
 * Effect collects all internal subscriptions, and provides `destroy()` methods
 * unsubscribe from them and deactivate the effect.
 */
export type Effect<Event, Result = void, ErrorType = Error> = Controller<EffectState<Event, Result, ErrorType> & {
    handle: (source: Action<Event> | Observable<Event> | Query<Event>) => Subscription;
}>;
/**
 * Creates `Effect` from the provided handler.
 *
 * @example
 * ```ts
 * const sumEffect = createEffect<{a: number, b: number}, number>((event) => {
 *   return a + b;
 * });
 * ```
 */
export declare function createEffect<Event = void, Result = void, ErrorType = Error>(handler: EffectHandler<Event, Result>, options?: EffectOptions<Event, Result>): Effect<Event, Result, ErrorType>;
